package edu.ucsd.act.apolrecruitfeed;

import java.io.File;
import java.io.FileReader;
import java.util.Properties;

/**
 * This class translates UCSD demographics code to corresponding UCI codes.
 * The basis for translations are maintained in properties files so that
 * mapping can be changed on the fly with having to modify Java code.
 * 
 * The constructor reads the properties files in to maps for improved runtime
 * performance.
 * 
 * @author David Little
 */ 
class CodeTranslator {
    
    private Properties genderMap;
    private Properties ethnicityMap;
    
    public CodeTranslator() throws Exception {
        ethnicityMap = new Properties();
        ethnicityMap.load(new FileReader(System.getProperty("config.dir") 
                + File.separator + "ethnicity.properties"));

        genderMap = new Properties();
        genderMap.load(new FileReader(System.getProperty("config.dir") 
                + File.separator + "gender.properties"));
    }

    /**
     * Translate the UCSD data warehouse ethnicity code to the UCI ethnicity value.
     * 
     * @param ethnicityCode UCSD ethnicity code
     * @return UCI ethnicity value
     */
    protected String translateEthnicity(String ethnicityCode) {
        if (ethnicityCode == null) {
            ethnicityCode = genderMap.getProperty("default");
        }
        String ethnicityCategory = ethnicityMap.getProperty(ethnicityCode,
                ethnicityCategory = ethnicityMap.getProperty("default")); 
        return ethnicityCategory;
    }
    
    
    /**
     * Translate the UCSD data warehouse gender code to the UCI gender value.
     * 
     * @param genderCode UCSD genderCode
     * @return UCI gender value
     */
    protected String translateGender(String genderCode) {
        if (genderCode == null) {
            genderCode = genderMap.getProperty("default");
        }
        String genderCategory = genderMap.getProperty(genderCode,
                genderCategory = genderMap.getProperty("default")); 
        return genderCategory;
    }
    
}
