package edu.ucsd.act.apolrecruitfeed;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.NoSuchAlgorithmException;
import java.util.Scanner;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

/**
 * This class static methods for decrypting encrypted passwords
 * 
 * Extracted from edu.ucsd.act.datasource.jndi.digest.CryptoUtils
 * 
 * @author David Little
 */
public class Decrypt {

    private static final String AES = "AES";

    /**
     * Decrypts a string with the aid of a key file.
     * 
     * @param message string to decrypt
     * @param keyFile file containing decryption key
     * @return decrypted message
     * @throws GeneralSecurityException
     * @throws IOException
     */
    public static String decrypt(String message, File keyFile)
            throws GeneralSecurityException, IOException {
        SecretKeySpec sks = getSecretKeySpec(keyFile);
        Cipher cipher = Cipher.getInstance(AES);
        cipher.init(Cipher.DECRYPT_MODE, sks);
        byte[] decrypted = cipher.doFinal(hexStringToByteArray(message));
        return new String(decrypted);
    }

    /**
     * First step in decryption process: derive a key spec from a key file.
     * 
     * @param keyFile
     * @return secret key spec
     * @throws NoSuchAlgorithmException
     * @throws IOException
     */
    private static SecretKeySpec getSecretKeySpec(File keyFile)
            throws NoSuchAlgorithmException, IOException {
        byte[] key = readKeyFile(keyFile);
        SecretKeySpec sks = new SecretKeySpec(key, AES);
        return sks;
    }

    /**
     * Translate a key file into a byte array. 
     * 
     * @param keyFile
     * @return the corresponding byte array
     * @throws FileNotFoundException
     */
    private static byte[] readKeyFile(File keyFile)
            throws FileNotFoundException {
        Scanner scanner = new Scanner(keyFile).useDelimiter("\\Z");
        String keyValue = scanner.next();
        scanner.close();
        return hexStringToByteArray(keyValue);
    }

    /**
     * Convert a string to a byte array.
     * 
     * @param s the string to be converted
     * @return the corresponding byte array
     */
    private static byte[] hexStringToByteArray(String s) {
        byte[] b = new byte[s.length() / 2];
        for (int i = 0; i < b.length; i++) {
            int index = i * 2;
            int v = Integer.parseInt(s.substring(index, index + 2), 16);
            b[i] = (byte) v;
        }
        return b;
    }

}
